#!/usr/bin/env bash

source ~/.bashrc
source ~/.bash_profile

targetDir=".."
java_pkg="com.taobao.idlefish.flutterboost"
android_src_path="$targetDir/android/src/main/java/com/taobao/idlefish/flutterboost"

gen_dir="services_gen"
ios_dir="$targetDir/ios/Classes/Messaging/Generated"

rm -R $gen_dir
mkdir $gen_dir

function install_service(){

	local service_name=$1
	local service_path=$(realpath "./AIOServices/$service_name.json")

	echo "using config $service_path"

	./AIOCodeGen/AIOCodeGen -c $service_path -o $(realpath $gen_dir) --java-pkg=$java_pkg

	if [ ! -d "$ios_dir" ]; then
		mkdir "$ios_dir"
	fi

#copy dart source

	if [ ! -d "$targetDir/lib/AIOService/" ]; then
		mkdir "$targetDir/lib/AIOService"
	fi

	if [ ! -d "$targetDir/lib/AIOService/$service_name" ]; then
		mkdir "$targetDir/lib/AIOService/$service_name"
	fi

	if [ ! -d "$targetDir/lib/AIOService/$service_name/handlers" ]; then
		mkdir "$targetDir/lib/AIOService/$service_name/handlers"
	fi

	if [ ! -d "$targetDir/lib/AIOService/$service_name/service" ]; then
		mkdir "$targetDir/lib/AIOService/$service_name/service"
	fi

	cp  $gen_dir/$service_name/dart/service/*.dart "$targetDir/lib/AIOService/$service_name/service/"
	cp  -n $gen_dir/$service_name/dart/handlers/*.dart "$targetDir/lib/AIOService/$service_name/handlers/"



#copy ios source
	if [ ! -d "$ios_dir/$service_name" ]; then
		mkdir "$ios_dir/$service_name"
	fi

	if [ ! -d "$ios_dir/$service_name/handlers" ]; then
		mkdir "$ios_dir/$service_name/handlers"
	fi

	if [ ! -d "$ios_dir/$service_name/service" ]; then
		mkdir "$ios_dir/$service_name/service"
	fi

	#copy service code override all.
	cp  $gen_dir/$service_name/oc/service/* "$ios_dir/$service_name/service/"
	#copy hanlders do not override because handlers could be modified.
	cp -n $gen_dir/$service_name/oc/handlers/* "$ios_dir/$service_name/handlers/"


#copy Android source

	if [ ! -d "$android_src_path/" ]; then
		mkdir "$android_src_path/"
	fi

	if [ ! -d "$android_src_path/$service_name" ]; then
		mkdir "$android_src_path/$service_name"
	fi

	cp  $gen_dir/$service_name/java/service/* "$android_src_path/$service_name/"
	cp -n $gen_dir/$service_name/java/handlers/*  "$android_src_path/$service_name/"


	echo "service : $service_name installed"
}

service_list=""

# Yaml support
# Parse yml files into json first
for file in AIOServicesYaml/*.yml; do
	filename=$(basename -- "$file")
	extension="${filename##*.}"
	filename="${filename%.*}"
	echo "pasing yml $filename"
    AIOCodeGen/yaml2json_darwin_amd64 < AIOServicesYaml/$filename.yml > AIOServices/$filename.json
done

for file in AIOServices/*.json; do
	filename=$(basename -- "$file")
	extension="${filename##*.}"
	filename="${filename%.*}"
	echo "installing $filename"
    install_service "$filename"
    service_list="$service_list,$filename"
done


# install java loader
echo "service list: $service_list"

# Generate service loaders. java and dart
./AIOCodeGen/AIOCodeGen -o $(realpath $gen_dir) --java-pkg=$java_pkg --java-loader=$service_list

# Copy java loader
if [ ! -d "$android_src_path/loader/" ]; then
		mkdir "$android_src_path/loader/"
fi
cp $gen_dir/java/loader/* "$android_src_path/loader/"


# Copy dart loader
if [ ! -d "$targetDir/lib/AIOService/loader" ]; then
		mkdir "$targetDir/lib/AIOService/loader"
fi

cp $gen_dir/dart/loader/* "$targetDir/lib/AIOService/loader"


# go back to previous dir
cd -
