package com.taobao.idlefish.flutterboost.containers;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Color;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.ProgressBar;

import com.taobao.idlefish.flutterboost.BoostFlutterView;
import com.taobao.idlefish.flutterboost.Debuger;

abstract public class FlutterViewStub extends FrameLayout {

    public static final Handler sHandler = new ProcessHandler(Looper.getMainLooper());

    protected Bitmap mBitmap;
    protected ImageView mSnapshot;
    protected FrameLayout mStub;
    protected View mCover;
    protected View mSplashScreenView;

    public FlutterViewStub(Context context) {
        super(context);

        mStub = new FrameLayout(context);
        mStub.setBackgroundColor(Color.WHITE);
        addView(mStub, new FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        mSnapshot = new ImageView(context);
        mSnapshot.setScaleType(ImageView.ScaleType.FIT_CENTER);
        mSnapshot.setLayoutParams(new FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        mCover = createFlutterInitCoverView();
        addView(mCover, new FrameLayout.LayoutParams(
                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));

        final BoostFlutterView flutterView = getBoostFlutterView();
        if (!flutterView.firstFrameCalled()) {
            mSplashScreenView = createSplashScreenView();
            addView(mSplashScreenView, new FrameLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        }
    }

    public void onContainerShown() {
        Debuger.log("onContainerShown");
        sHandler.postDelayed(new Runnable() {
            @Override
            public void run() {
                if (mSplashScreenView != null) {
                    FlutterViewStub.this.removeView(mSplashScreenView);
                    mSplashScreenView = null;
                }

                if (mCover != null) {
                    FlutterViewStub.this.removeView(mCover);
                }

                if (mSnapshot.getParent() != FlutterViewStub.this) return;

                FlutterViewStub.this.removeView(mSnapshot);
                mSnapshot.setImageBitmap(null);
                if (mBitmap != null && !mBitmap.isRecycled()) {
                    mBitmap.recycle();
                    mBitmap = null;
                }

                //做一次截屏可以避免黑屏，神奇...
                final BoostFlutterView flutterView = (BoostFlutterView) mStub.getChildAt(0);
                if (flutterView != null) {
                    final Bitmap bitmap = flutterView.getBitmap();
                    if (bitmap != null && !bitmap.isRecycled()) {
                        bitmap.recycle();
                    }
                }

                getBoostFlutterView().scheduleFrame();
                getBoostFlutterView().requestFocus();
                getBoostFlutterView().invalidate();
            }
        }, 160);
    }

    public void onContainerHidden() {
        //Debuger.log("onContainerHidden");
    }

    public void snapshot() {
        if (mStub.getChildCount() <= 0) return;
        if (mSnapshot.getParent() != null) return;

        BoostFlutterView flutterView = (BoostFlutterView) mStub.getChildAt(0);

        mBitmap = flutterView.getBitmap();
        if (mBitmap != null && !mBitmap.isRecycled()) {
            mSnapshot.setImageBitmap(mBitmap);
            addView(mSnapshot);
        }
    }

    public void attachFlutterView(final BoostFlutterView flutterView) {
        if (flutterView.getParent() != mStub) {
            sHandler.removeMessages(ProcessHandler.MSG_DETACH);

            Debuger.log("attachFlutterView");

            if (flutterView.getParent() != null) {
                ((ViewGroup) flutterView.getParent()).removeView(flutterView);
            }

            mStub.addView(flutterView, new FrameLayout.LayoutParams(
                    ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
        }
    }


    public void detachFlutterView() {
        if (mStub.getChildCount() <= 0) return;

        final BoostFlutterView flutterView = (BoostFlutterView) mStub.getChildAt(0);

        if (flutterView == null) return;

        Debuger.log("detachFlutterView");

        if (mSnapshot.getParent() == null) {
            mBitmap = flutterView.getBitmap();
            if (mBitmap != null && !mBitmap.isRecycled()) {
                mSnapshot.setImageBitmap(mBitmap);
                addView(mSnapshot);
            }
        }

        Message msg = new Message();
        msg.what = ProcessHandler.MSG_DETACH;
        msg.obj = new Runnable() {
            @Override
            public void run() {
                if (flutterView.getParent() != null && flutterView.getParent() == mStub) {
                    mStub.removeView(flutterView);
                }
            }
        };
        sHandler.sendMessage(msg);
    }

    public void destroy() {
        removeAllViews();
        mSnapshot.setImageBitmap(null);
        if (mBitmap != null && !mBitmap.isRecycled()) {
            mBitmap.recycle();
            mBitmap = null;
        }
    }

    protected View createFlutterInitCoverView() {
        View initCover = new View(getContext());
        initCover.setBackgroundColor(Color.WHITE);
        return initCover;
    }

    protected View createSplashScreenView() {
        return new ProgressBar(getContext());
    }

    abstract protected BoostFlutterView getBoostFlutterView();

    public static class ProcessHandler extends Handler {

        static final int MSG_DETACH = 175101;

        ProcessHandler(Looper looper) {
            super(looper);
        }

        @Override
        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            if (msg.obj instanceof Runnable) {
                Runnable run = (Runnable) msg.obj;
                run.run();
            }
        }
    }
}
