//
//  FLBFlutterApplication.m
//  flutter_boost
//
//  Created by Jidong Chen on 2018/10/16.
//

#import "FLBFlutterApplication.h"
#import "FlutterBoost.h"
#import "FLBFlutterViewContainerManager.h"
#import "FLBViewProviderFactory.h"

@interface FLBFlutterApplication()
@property (nonatomic,strong) FLBFlutterViewContainerManager *manager;
@property (nonatomic,strong) id<FLBFlutterViewProvider> viewProvider;

@property (nonatomic,strong) NSMutableDictionary *pageBuilders;
@property (nonatomic,copy) FLBPageBuilder defaultPageBuilder;
@property (nonatomic,assign) BOOL isRendering;
@property (nonatomic,assign) BOOL isRunning;
@end


@implementation FLBFlutterApplication
static bool sIsRunning = false;

+ (FLBFlutterApplication *)sharedApplication
{
    static FLBFlutterApplication *instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        instance = [self new];
    });
    return instance;
}

- (BOOL)isRunning
{
    return _isRunning;
}

- (void)startFlutterWithPlatform:(id<FLBPlatform>)platform
                         onStart:(void (^)(FlutterViewController * _Nonnull))callback
{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _platform = platform;
        _viewProvider = [[FLBViewProviderFactory new] createViewProvider];
        [_viewProvider resume];
        _isRendering = YES;
        _isRunning = YES;
        if(callback) callback(_viewProvider.viewController);
    });
}

- (instancetype)init
{
    if (self = [super init]) {
        _manager = [FLBFlutterViewContainerManager new];
        _pageBuilders = [NSMutableDictionary new];
    }
    return self;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (UIView *)flutterView
{
    return [self flutterViewController].view;
}

- (FlutterViewController *)flutterViewController
{
    return self.viewProvider.viewController;
}


- (BOOL)contains:(FLBFlutterViewContainer  *)vc
{
    return [_manager contains:vc];
}

- (void)addUniqueViewController:(FLBFlutterViewContainer  *)vc
{
    return [_manager addUnique:vc];
}

- (void)removeViewController:(FLBFlutterViewContainer  *)vc
{
    return [_manager remove:vc];
}




- (BOOL)isTop:(NSString *)pageId
{
    return [_manager.peak isEqual:pageId];
}

- (void)pause
{
    if(!_isRendering) return;
    
    [self.viewProvider pause];
    
    _isRendering = NO;
    
#if DEBUG
    NSLog(@"Flutter appication paused");
#endif
}

- (void)resume
{
    if(_isRendering) return;
    
    [self.viewProvider resume];
    
    _isRendering = YES;
    
#if DEBUG
    NSLog(@"Flutter appication resumed");
#endif
}

- (void)inactive
{
    [self.viewProvider inactive];
}



@end
