
#import "FlutterBoostPlugin.h"
#import "FLBFlutterApplication.h"
#import "FLBResultMediator.h"

@interface FlutterBoostPlugin()
@property (nonatomic,strong) FLBResultMediator *resultMediator;
@end

@implementation FlutterBoostPlugin

+ (instancetype)sharedInstance
{
    static id _instance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _instance = [self.class new];
    });
    
    return _instance;
}

- (instancetype)init
{
    if (self = [super init]) {
        _resultMediator = [FLBResultMediator new];
    }
    
    return self;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
  FlutterMethodChannel* channel = [FlutterMethodChannel
      methodChannelWithName:@"flutter_boost"
            binaryMessenger:[registrar messenger]];
  FlutterBoostPlugin* instance = [self.class sharedInstance];
  [registrar addMethodCallDelegate:instance channel:channel];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
  if ([@"getPlatformVersion" isEqualToString:call.method]) {
    result([@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]]);
  } else {
    result(FlutterMethodNotImplemented);
  }
}


- (void)startFlutterWithPlatform:(id<FLBPlatform>)platform
                         onStart:(void (^)(FlutterViewController *))callback;
{
    [FLBFlutterApplication.sharedApplication startFlutterWithPlatform:platform
                                                              onStart:callback];
}

- (BOOL)isRunning
{
    return [FLBFlutterApplication.sharedApplication isRunning];
}

- (FlutterViewController *)currentViewController
{
    return [[FLBFlutterApplication sharedApplication] flutterViewController];
}

- (void)onResultForKey:(NSString *)vcId resultData:(NSDictionary *)resultData
{
    [_resultMediator onResultForKey:vcId resultData:resultData];
}

- (void)setResultHandler:(void (^)(NSString *, NSDictionary *))handler forKey:(NSString *)vcid
{
    [_resultMediator setResultHandler:handler forKey:vcid];
}

- (void)removeHandlerForKey:(NSString *)vcid
{
    [_resultMediator removeHandlerForKey:vcid];
}

@end
