//
//  FMStackCacheObjectImg.m
//  Pods
//
//  Created by Jidong Chen on 2018/8/2.
//

#import "FLBStackCacheObjectImg.h"

@interface FLBStackCacheObjectImg()
@property (nonatomic,strong) UIImage *image;
@end

@implementation FLBStackCacheObjectImg

- (instancetype)initWithImage:(UIImage *)image
{
    if (self = [super init]) {
        _image = image;
    }
    
    return self;
}



+ (BOOL)loadFromFileWithKey:(NSString *)key
                      queue:(dispatch_queue_t)queue
                      cache:(FLBStackCache *)cache
                 completion:(void (^)(NSError *, id<FLBStackCacheObject>))completion
{
    dispatch_async(queue, ^{
        UIImage *image = [[UIImage alloc] initWithContentsOfFile:[self filePathByKey:key dirPath:cache.cacheDir]];
        if (completion) {
            if (image) {
                FLBStackCacheObjectImg *ob = [[FLBStackCacheObjectImg alloc] initWithImage:image];
                ob.key = key;
                completion(nil,ob);
            }else{
                completion([NSError new],nil);
            }
        }
    });
    
    return YES;
}

+ (NSString *)filePathByKey:(NSString *)key dirPath:(NSString *)cacheDir
{
    key = [key stringByReplacingOccurrencesOfString:@"/" withString:@""];
    key = [key stringByReplacingOccurrencesOfString:@":" withString:@""];
    NSString *path = [cacheDir stringByAppendingPathComponent:key];
    return path;
}

- (BOOL)writeToFileWithKey:(NSString *)key
                     queue:(dispatch_queue_t)queue
                     cache:(FLBStackCache *)cache
                completion:(void (^)(NSError *, NSString *))completion
{
    if(!_image){
        return NO;
    }
    
    dispatch_async(queue, ^{
        NSData *imgData = UIImagePNGRepresentation(self.image);
        NSString *filePath = [FLBStackCacheObjectImg filePathByKey:key dirPath:cache.cacheDir];
        [imgData writeToFile:filePath atomically:YES];
        if (completion) {
            completion(nil,key);
        }
    });
    
    return YES;
}

@end
