import 'logger.dart';

class Tracer {
  static final Tracer singleton = Tracer();

  final Map<String, Record> _records = <String, Record>{};
  Tracer();

  static String mark(String unique, String tag) {
    Record record = singleton._records[unique];

    if (record == null) {
      record = Record()
        ..unique = unique
        ..marks = <Mark>[];
      singleton._records[unique] = record;
    }

    record.marks.add(Mark(tag, DateTime.now()));

    return record.toString();
  }

  static void markAndLog(String unique, String tag) {
    Logger.log(mark(unique, tag));
  }

  static String dump(String unique) => singleton._records[unique]?.toString();
}

class Record {
  String unique;
  List<Mark> marks;

  @override
  String toString() {
    if (marks == null || marks.isEmpty) {
      return '';
    }

    if (marks.length == 1) {
      return marks.first.tag;
    }

    Mark least = marks.first;
    String info = 'trace<$unique>#${least.tag}';
    for (int i = 1; i < marks.length; i++) {
      final Mark mark = marks[i];
      info =
          '$info=${mark.timeStamp.millisecond - least.timeStamp.millisecond}ms=>${mark.tag}';
      least = mark;
    }

    return info;
  }
}

class Mark {
  String tag;
  DateTime timeStamp;

  Mark(this.tag, this.timeStamp);
}
